package cn.com.qmth.stmms.ms.core.utils;

import cn.com.qmth.stmms.ms.commons.config.DbConfig;
import cn.com.qmth.stmms.ms.core.domain.Organization;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.repository.OrganizationRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import org.apache.commons.io.FileUtils;
import org.apache.http.client.utils.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.*;
import java.nio.charset.Charset;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @Date: 2021/5/31.
 */
@Component
public class DbBackupUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(DbBackupUtils.class);

    ExecutorService service = Executors.newFixedThreadPool(1);

    @Autowired
    DbConfig dbConfig;

    @Autowired
    OrganizationRepo organizationRepo;

    @Autowired
    WorkRepo workRepo;

    public void startBackup(Long workId, String stage) {
        Work work = workRepo.getOne(workId);
        if (Objects.nonNull(work)) {
            Organization organization = organizationRepo.getOne(work.getOrganizationId());
            if (Objects.nonNull(organization)) {
                String code = organization.getAbbreviation();
                service.submit(dbBackUp(workId, code, stage));
            }
        }
    }

    public Runnable dbBackUp(Long workId, String code, String stage) {
        return () -> {
            String path = "beifen";
            String pathSql = code + "-" + workId + "-" + stage + "-" + DateUtils.formatDate(new Date(), "yyyyMMddHHmmss") + ".sql";
            File fileSql = new File(path, pathSql);
            try {
                if (!fileSql.getParentFile().exists()) {
                    fileSql.getParentFile().mkdirs();
                }
                //创建备份sql文件
                if (!fileSql.exists()) {
                    fileSql.createNewFile();
                }

//                String mysqldump = getMysqldump(fileSql);
                //mysqldump -hlocalhost -uroot -p123456 db > /home/back.sql
                StringBuffer sb = new StringBuffer();
                sb.append("cmd /c ");
                sb.append("mysqldump");
                sb.append(" -h " + dbConfig.getHost());
                sb.append(" -u" + dbConfig.getUserName());
                sb.append(" -p" + dbConfig.getPassword());
                sb.append(" -P" + dbConfig.getPort());
                sb.append(" " + dbConfig.getDbName() + " >");
                sb.append(fileSql.getPath());
                LOGGER.info("cmd命令为：{}", sb.toString());
                Runtime runtime = Runtime.getRuntime();
                runtime.exec(sb.toString());
                LOGGER.info("备份成功!");
            } catch (Exception e) {
                LOGGER.info("备份失败！原因：{}", e.getMessage());
            }
        };
    }

    public static String getMysqldump(File fileSql) throws IOException, InterruptedException {
        Runtime runtime = Runtime.getRuntime();
        Process process = runtime.exec("where mysqldump");
        BufferedReader br = new BufferedReader(new InputStreamReader(process.getInputStream(), Charset.forName("GBK")));
        String line = null;
        String result = "";
        while ((line = br.readLine()) != null) {
            System.out.println(line);
            result += line;
        }
        br.close();
        process.waitFor();
        File destFile = new File(fileSql.getParent(), "mysqldump.exe");
        if (result.contains("mysqldump")) {
            if (destFile.exists()) {
                destFile.delete();
            }
            FileUtils.copyFile(new File(result), destFile);
        } else {
            try {
                FileOutputStream out = new FileOutputStream(fileSql.getAbsolutePath(), true);
                String str = "没有找到mysqldump命令";
                out.write(str.getBytes());
                out.close();
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }
            throw new RuntimeException("没有找到mysqldump命令");
        }
        return destFile.getParent() + File.separator + "mysqldump";
    }
}
